# -*- coding: iso_8859_1 -*-

from aspylib import astro
import numpy as np
import matplotlib.pyplot as plt

#------------------------------------------------------------------------------------
#	Purpose
#------------------------------------------------------------------------------------
#
#	This script calculates the uniformity of the sky background over a serie of images
#	- the deviation from a uniform sky background is computed at the 4 corners of the image
#	- the variation of this quantity is calculated and plotted
#
#------------------------------------------------------------------------------------
#	Input parameters	
#------------------------------------------------------------------------------------

Folder = u"C:\\images\\"
Imagelist = [Folder + "images-coaligned-" + str(i+1) + ".fit" for i in range(50)]

#------------------------------------------------------------------------------------
#	Script
#------------------------------------------------------------------------------------

median1 = []
median2 = []
median3 = []
median4 = []

sizeX = astro.get_headervalues([Imagelist[0]],'NAXIS2')[0]
sizeY = astro.get_headervalues([Imagelist[0]],'NAXIS1')[0]

for i in range(len(Imagelist)):
	imagei = astro.get_imagedata([Imagelist[i]])
	
	#calculating median values
	m1 = np.ma.median(imagei[100:200,100:200].flatten())
	m2 = np.ma.median(imagei[sizeX-200:sizeX-100,100:200].flatten())
	m3 = np.ma.median(imagei[100:200,sizeY-200:sizeY-100].flatten())
	m4 = np.ma.median(imagei[sizeX-200:sizeX-100,sizeY-200:sizeY-100].flatten())
	moy = (m1+m2+m3+m4)/4.
	
	#deviation from uniform sky background
	median1.append(m1-moy)
	median2.append(m2-moy)
	median3.append(m3-moy)
	median4.append(m4-moy)

#variations of the above quantity
median1 = median1 - np.mean(median1)
median2 = median2 - np.mean(median2)
median3 = median3 - np.mean(median3)
median4 = median4 - np.mean(median4)
	
plt.figure()
plt.plot(median1)
plt.plot(median2)
plt.plot(median3)
plt.plot(median4)
plt.title("Variations of the image non-uniformity")
plt.grid()
plt.show()
raw_input('Press Enter to close')