# -*- coding: iso_8859_1 -*-

from aspylib import astro, license, conf
import numpy as np
import time, os, datetime, warnings
import matplotlib.pyplot as plt
import matplotlib.font_manager as fm
from matplotlib.widgets import Button
from matplotlib import cm
plt.ion()

#------------------------------------------------------------------------------------
#	Purpose 
#------------------------------------------------------------------------------------
#
#	Opens the text file created by the script "photosearch_run.py" and allows the user to display all lightcurves one by one, and look for variable stars.
#	For more explanations see: http://www.aspylib.com/doc/using_cdr.html
#
#------------------------------------------------------------------------------------
#		Input parameters
#------------------------------------------------------------------------------------

#--- input frames ---
Folder = u"C:\\images\\"
File = Folder + "target_03_magnitudes.txt"
Image = Folder + 'image-calibrated-1.fit'

mode = 1
#mode = 1 --> plot is magnitude / global noise
#mode = 2 --> plot is local noise / global noise

#------------------------------------------------------------------------------------
#		Script
#------------------------------------------------------------------------------------
#*********************************************************************************************************
#		Function to read inputs
#*********************************************************************************************************

def load_data(filename, string_start, string_end, colstart=0):

	#display file name
	file_path = filename.split(conf.sep)
	file_core = file_path[len(file_path)-1]
	print "... loading data in %s ..." % file_core

	#open file and reads data
	f=open(filename, 'r')
	try:
		data = []
		flag = 0
		notlist = ["\n", "#"]
		for line in f:
			if flag:
				if line[:len(string_end)]==string_end:		
					flag = 0					

			if flag:
				datalist = line.split('\t')
				if datalist[0] not in notlist:
					#stores data in array
					x = []
					for i,dd in enumerate(datalist):
						if (i>=colstart)and(dd!='\n'):
							if (dd[:8]=='1.#QNAN0')or(dd[:2]=='--')or(dd[:3]=='nan'):
								x.append(np.nan)
							else:
								x.append(dd.replace('\n',''))
					data.append(x)

			if (1-flag):
				if line[:len(string_start)]==string_start:	
					flag = 1
	finally:
		f.close()
	try:
		return np.array(data,dtype=np.float64)
	except:
		return np.array(data)

#*********************************************************************************************************
#		Reading information
#*********************************************************************************************************

print
data = load_data(File,string_start="#--- JULIAN DAY",string_end="\n")
print "number of images:",np.shape(data)[0]
print "measured objects:",np.shape(data)[1]-1
print

obj = load_data(File,string_start="#--- OBJECTS",string_end="\n",colstart=1)
astrom = load_data(File,string_start="#--- ASTROMETRY",string_end="\n",colstart=1)
usnob1 = load_data(File,string_start="#--- MATCHING",string_end="\n",colstart=1)
stats = load_data(File,string_start="#--- STATISTICS",string_end="\n",colstart=1)

print
print "nb of objects with given quality:"
print "quality 0:",np.sum(stats[3,:]==0)
print "quality 1:",np.sum(stats[3,:]==1)
print "quality 2:",np.sum(stats[3,:]==2)
print "quality 3:",np.sum(stats[3,:]==3)
print "quality 4:",np.sum(stats[3,:]==4)
print

ind0 = list(np.where(np.array(stats[3,:])==0)[0])
ind1 = list(np.where(np.array(stats[3,:])==1)[0])
ind2 = list(np.where(np.array(stats[3,:])==2)[0])
ind3 = list(np.where(np.array(stats[3,:])==3)[0])
ind4 = list(np.where(np.array(stats[3,:])==4)[0])

flagplot = stats[3,:] <= 4

#--- calculating magnitude constant (draft) ---
refstars = load_data(File,string_start="#--- REFERENCE",string_end="\n",colstart=1)
refstars = np.array([refstars[2], refstars[5], refstars[8]], dtype=np.int)

#*********************************************************************************************************
#		Plotting stats information
#*********************************************************************************************************

xy = []

fig = plt.figure()
ax = plt.subplot(111)
if mode==1:
	if len(ind0)>0:
		ax.scatter(stats[0,ind0],stats[1,ind0],color='g', s=10, label='< 0.05')
	if len(ind1)>0:
		ax.scatter(stats[0,ind1],stats[1,ind1],color='#bbfe00', s=10, label='< 0.10')
	if len(ind2)>0:
		ax.scatter(stats[0,ind2],stats[1,ind2],color='y', s=10, label='< 0.25')
	if len(ind3)>0:
		ax.scatter(stats[0,ind3],stats[1,ind3],color='#fe8c00', s=10, label='< 0.50')
	if len(ind4)>0:
		ax.scatter(stats[0,ind4],stats[1,ind4],color='r', s=10, label='> 0.50')
	plt.xlabel('instrumental magnitude')
	plt.ylabel('global magnitude noise')
elif mode==2:
	if len(ind0)>0:
		ax.scatter(stats[2,ind0],stats[1,ind0],color='g', s=10, label='< 0.05')
	if len(ind1)>0:
		ax.scatter(stats[2,ind1],stats[1,ind1],color='#bbfe00', s=10, label='< 0.10')
	if len(ind2)>0:
		ax.scatter(stats[2,ind2],stats[1,ind2],color='y', s=10, label='< 0.25')
	if len(ind3)>0:
		ax.scatter(stats[2,ind3],stats[1,ind3],color='#fe8c00', s=10, label='< 0.50')
	if len(ind4)>0:
		ax.scatter(stats[2,ind4],stats[1,ind4],color='r', s=10, label='> 0.50')
	plt.xlabel('local magnitude noise')
	plt.ylabel('global magnitude noise')
plt.grid()
box = ax.get_position()
ax.set_position([box.x0, box.y0, box.width * 0.87, box.height])
leg = plt.legend(title = r'$\mathrm{\mathsf{max(|m_{i+1}-m_i|)}}$', prop=fm.FontProperties(size=11), loc='center left', bbox_to_anchor=(1.0, 0.5))
leg.get_title().set_fontproperties(fm.FontProperties(size=13))
plt.draw()

#*********************************************************************************************************
#		Plotting lightcurve and star image
#*********************************************************************************************************

def export_cdrcdl(event):
	f = open(Folder+"object_"+str(index+1)+"_cdrcdl.txt", 'w')
	# f.write("-------------------------------------------------\n")
	# f.write("      Object "+str(index+1)+"\n")
	# f.write("-------------------------------------------------\n\n")
	f.write(strinfo)
	f.write("\n\n")
	f.write("Column 1: geocentric Julian date at mid-exposure\n")
	f.write("Column 2: differential magnitude, differs from true magnitude by a constant offset\n")
	f.write("Column 3: magnitude noise\n")
	f.write("------------------------------------------------------------------------\n")
	f.write("FMT DVv\n")
	f.write("NOM star_"+str(index+1)+"\n")
	if usnob1[1,index]=='-':
		f.write(";alpha="+(astrom[0,index]).replace(":"," ")+", delta="+(astrom[1,index]).replace(":"," ")+"\n")
	else:
		f.write(";USNO-B1 "+usnob1[1,index]+" alpha="+(usnob1[2,index]).replace(":"," ")+", delta="+(usnob1[3,index]).replace(":"," ")+"\n")
	f.write("MES\n")
	for i in range(len(data)):
		f.write("%(JJ).6f\t%(mag).6f\t%(sigma).6f\n" % {'JJ':data[i,0],'mag':data[i,index+1],'sigma':stats[2,index]})
	f.close()
	print "object "+str(index+1)+" exported to .txt file"

warnings.simplefilter("ignore") 
fig2 = plt.figure(figsize=(8,10))
ax1 = fig2.add_subplot(211)
ax2 = fig2.add_subplot(212)
fig2.subplots_adjust(bottom=0.11)
fig2.subplots_adjust(top=0.95)
but1 = Button(plt.axes([0.37,0.015,0.3,0.04]),'Export to CdR-CdL file')
but1.on_clicked(export_cdrcdl)

try:
	image = astro.get_imagedata([Image],text_display=False)
	image = np.array(image,dtype=np.int)
except:
	image = np.zeros((1,1))

while 1:
	def on_click(event): 
		global ax1, ax2, index, strinfo
		if event.inaxes is not None: 
			if event.button==3: 
				xy = [event.xdata,event.ydata]
				
				#--------- finding object -----------
				if mode==1:
					dist = np.sqrt((xy[0]-stats[0,:]*flagplot)**2.+25.*(xy[1]-stats[1,:]*flagplot)**2.)
				elif mode==2:
					dist = np.sqrt((xy[0]-stats[2,:]*flagplot)**2.+(xy[1]-stats[1,:]*flagplot)**2.)
				index = np.argsort(dist)[0]

				#--------- printing object information ---------
				strinfo = ""
				strinfo += "------------------------------------------------------------------------\n"
				strinfo += "          object "+str(index+1)+"\n"
				strinfo += "------------------------------------------------------------------------\n"
				strinfo += "AsPyLib (vert,horiz) position : %(x).3f, %(y).3f" % {'x':obj[1,index],'y':obj[2,index]}+"\n"
				strinfo += "Prism (X,Y) position          : %(x).3f, %(y).3f" % {'x':1+obj[2,index],'y':1+obj[1,index]}+"\n"
				strinfo += "mean mag     : %.5f" % stats[0,index]+"\n"
				strinfo += "local noise  : %.5f" % stats[1,index]+"\n"
				strinfo += "global noise : %.5f" % stats[2,index]+"\n"
				strinfo += "quality flag : %.0f" % stats[3,index]+"\n"
				strinfo += "------------------------------------------------------------------------\n"
				if np.sum(np.shape(astrom))>0:
					strinfo += "measured position (alpha,delta) : " + astrom[0,index] +" "+ astrom[1,index]+"\n"
					strinfo += "------------------------------------------------------------------------\n"
					strinfo += "best match USNO-B1.0\n"
					strinfo += "- distance (arcsec)      : "+ usnob1[0,index]+"\n"
					strinfo += "- name                   : "+ usnob1[1,index]+"\n"
					strinfo += "- position (alpha,delta) : "+ usnob1[2,index]+" "+ usnob1[3,index]+"\n"
					strinfo += "- mag (B1,B2,R1,R2)      : "+ str(usnob1[4,index]) +" "+ str(usnob1[5,index]) +" "+ str(usnob1[6,index]) +" "+ str(usnob1[7,index])+"\n"
					strinfo += "------------------------------------------------------------------------"

				print
				print strinfo

				#--------- displaying star image ---------
				plt.sca(ax1)
				plt.cla()
				med = astro.get_median(image)[0]
				dsig = np.std(image.flatten())
				ax1.imshow(image, interpolation='nearest',cmap=cm.gray,origin='lower',vmin = med-0.5*dsig, vmax =med+2*dsig, aspect='equal', extent=None)
				ax1.set_xlim([obj[2,index]-100,obj[2,index]+100])
				ax1.set_ylim([obj[1,index]-100,obj[1,index]+100])
				ax1.scatter(obj[2,index],obj[1,index],s=200,marker='o',color='r',facecolors='none')
				
				#--------- plotting lightcurve and star image ---------
				plt.sca(ax2)
				plt.cla()
				plt.plot(data[:,0],data[:,index+1],label="object "+str(index+1))
				mask = ~(np.isnan(data[:,index+1]))
				mini = np.min(data[mask,index+1])
				maxi = np.max(data[mask,index+1])
				if maxi-mini<0.5:
					av = (maxi+mini)*0.5
					plt.ylim([av-0.25,av+0.25])
				plt.grid()
				plt.legend()
				limites = ax2.get_ylim()
				if limites[-1]>limites[0]:
					ax2.set_ylim(limites[::-1])
				plt.draw()
		else:
			fig.canvas.mpl_disconnect(cid) 
			fig.canvas.stop_event_loop_default() 		

	cid = fig.canvas.mpl_connect('button_press_event', on_click) 
	fig.canvas.start_event_loop_default() 

warnings.resetwarnings() 

raw_input("finished")