# -*- coding: iso_8859_1 -*-

from aspylib import cdr, astro
import numpy as np
import matplotlib.pyplot as plt

#------------------------------------------------------------------------------------
#	Purpose
#------------------------------------------------------------------------------------
#
#	Reads a list of lightcurve files (with CourbRot format) and makes a Fourier fit.
#	The following are calculated:
#	- the lightcurve period
#	- the min-to-max amplitude
#	- date of min and max magnitudes
#	- a phase ephemeris
#	The time delays due to light propagation are corrected. For stars all the dates are heliocentric, for asteroids the dates are corrected for light propagation from asteroid to Earth
#
#------------------------------------------------------------------------------------
#	Input parameters
#------------------------------------------------------------------------------------

#--- input files ---
Folder = u"C:\\Mesures\\"
Filelist = ["File_1.txt", \
			"File_2.txt", \
			"File_3.txt"]

Filelist = [Folder + x for x in Filelist]

#fit parameters
T0 = 2456000.5			#reference time (this parameter defines the time when phase = 0. It is also needed in the calculation to avoid too large numbers for the epochs)
period = 0.4477			#starting period for the optimisation loop
nb_harmo = 8			#number of Fourier harmonics for fitting the lightcurve

#type of target (either "asteroid" or "star")
target = "asteroid"		
#target = "star"

#for time correction of variable star
alpha = astro.hours_to_decdegrees('10:33:25.79')
delta = astro.degrees_to_decdegrees('+17:34:27.4')

#for time correction of asteroid
file_DistanceToEarth = Folder + "dist.dat"

#for plotting the lightcurve
title = "Target\nLightcurve with Fourier fit"

#for phase ephemeris
eph_start = 2455822.5
eph_end = 2455842.5
eph_nbpts = 241

#------------------------------------------------------------------------------------
#	Script
#------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------
#	loading data
#------------------------------------------------------------------------------------

alldata = cdr.load_magdata(Filelist)

#------ time correction (choose asteroid or variable star) ------
if target == "asteroid":
	alldata = cdr.time_correction(alldata, 1, file_DistanceToEarth)			#time correction for asteroid
else:
	alldata = cdr.time_correction(alldata, 2, [alpha,delta])				#time correction for variable star

cdr.plot_magdata(alldata, 'Measurements')

#------------------------------------------------------------------------------------
#	fitting data
#------------------------------------------------------------------------------------

popt, pcov, period, dperiod, amplitude, maxdate, mindate = cdr.fit_CDR(alldata, period, T0, nb_harmo = nb_harmo, show_results = True, show_plot = True)
plt.ylim([0.401,-0.401])
plt.title(title)
plt.show()

#------------------------------------------------------------------------------------
#	generating phase ephemerid
#------------------------------------------------------------------------------------

JJ = np.linspace(eph_start, eph_end, eph_nbpts)
print
print "------ phase ephemerids ------"	
phase = (JJ-T0)/popt[0]-np.floor((JJ-T0)/popt[0])

for i,dd in enumerate(JJ):
	print dd, phase[i]

raw_input("Press Enter")
