# -*- coding: iso_8859_1 -*-

from scipy import optimize
import numpy as np

def distorsion_law(p, xy):
	#transforms pixel coordinates (xy) into projected stereographic coordinates (proj)			
	return np.vstack([p[0] + p[1]*xy[:,0] + p[2]*xy[:,1] + p[3]*(xy[:,0]**2.) + p[4]*xy[:,0]*xy[:,1] + p[5]*(xy[:,1]**2.) +\
					p[6]*(xy[:,0]**3.) + p[7]*(xy[:,0]**2.)*xy[:,1] + p[8]*xy[:,0]*(xy[:,1]**2.) + p[9]*(xy[:,1]**3.), \
					p[10] + p[11]*xy[:,0] + p[12]*xy[:,1] + p[13]*(xy[:,0]**2.) + p[14]*xy[:,0]*xy[:,1] + p[15]*(xy[:,1]**2.) +\
					p[16]*(xy[:,0]**3.) + p[17]*(xy[:,0]**2.)*xy[:,1] + p[18]*xy[:,0]*(xy[:,1]**2.) + p[19]*(xy[:,1]**3.)]).transpose()
					
def distorsion_law_quadratic(p, xy):
	return np.vstack([p[0] + p[1]*xy[:,0] + p[2]*xy[:,1] + p[3]*(xy[:,0]**2.) + p[4]*xy[:,0]*xy[:,1] + p[5]*(xy[:,1]**2.), \
					p[6] + p[7]*xy[:,0] + p[8]*xy[:,1] + p[9]*(xy[:,0]**2.) + p[10]*xy[:,0]*xy[:,1] + p[11]*(xy[:,1]**2.)]).transpose()
					
def distorsion_law_linear(p, xy):
	return np.vstack([p[0] + p[1]*xy[:,0] + p[2]*xy[:,1], p[3] + p[4]*xy[:,0] + p[5]*xy[:,1]]).transpose()

def inverse_distorsion_law(p, proj):
	#transforms projected stereographic coordinates (proj) into pixel coordinates (xy)

	residual = lambda xy0,proj0: distorsion_law(p,np.tile(xy0,(1,1)))[0,:]-proj0

	xy = np.array(map(lambda proj0: optimize.fsolve(residual,[0.,0.],args=(proj0)), proj))

	return xy
					
def invert_simple_coefficients(p):
	#input = coefficients [scaling, phi, dx, dy] 
	#output = coefficients [scaling2, phi2, dx2, dy2] 
	#rotation angles (phi, phi2) in radians !
	
	scaling, phi, dx, dy = p
	
	scaling2 = 1./scaling
	phi2 = - phi
	dx2 = - scaling2 * (np.cos(phi)*dx + np.sin(phi)*dy)
	dy2 = - scaling2 * (-np.sin(phi)*dx + np.cos(phi)*dy)
	
	return [scaling2, phi2, dx2, dy2]

def transform_stars(param, xy):

	scaling, phi, dx, dy = param
	trans = map(lambda coord: [scaling*(coord[0]*np.cos(phi) - coord[1]*np.sin(phi)) + dx, \
							   scaling*(coord[0]*np.sin(phi) + coord[1]*np.cos(phi)) + dy],xy)
	return np.array(trans)

