# -*- coding: iso_8859_1 -*-

import warnings
import numpy as np
from vo.table import parse_single_table

def read_votable(filename, ordering = [False,''], remove_duplicates = [False, []]):
	"""
	---------------------
	Purpose
	Reads out a xml file with VOtable format, that was obtained with the Vizier or SkyBoT web service
	---------------------
	Inputs
	* filename (string) = specifies the xml file from which the data must be read out.
	It can be a full filename with path information, or a simple filename in which case the active directory is used.
	* ordering (list) = optional parameter, default is [False, '']. If ordering[0] is True, all the astronomical objects contained in the data are ordered in increasing order using the field ordering[1].
	For instance, if ordering = [True, 'B1mag'] then the output data is such that data['B1mag'] is in increasing order.
	* remove_duplicates (list) = optional parameter, default is [False,[]].
	If remove_duplicates[0] is True, then all astronomical objects having two or more occurrences with the same fields remove_duplicates[1][i] (with i=1,..,len(remove_duplicates[1])) are identified and all redundant occurrences are deleted.
	For instance, remove_duplicates = [True, ['RAJ2000', 'DEJ2000']] allows to make sure that we have only one object per (alpha, delta) position.
	---------------------
	Output (Numpy record array) = array that contains all the data. The columns get their names from both the ID and name attributes of the FIELD elements in the VOTABLE file.
	See online help: http://stsdas.stsci.edu/astrolib/vo/html/intro_table.html#using-vo-table
	---------------------
	"""
	
	#------ (1) Converting votable to Numpy array -----------------

	print
	print "--- reading existing .xml catalog ---"

	warnings.simplefilter("ignore")
	votable = parse_single_table(filename, pedantic=False)
	warnings.resetwarnings() 
	data = votable.array
	
	#------ (2) Ordering objects -----------------

	if ordering[0]:
		data = data[np.argsort(data[ordering[1]]),:]

	#------ (3) Removing duplicates -----------------

	if remove_duplicates[0]:
		data2 = data[remove_duplicates[1][0]]
		for i in range(1,len(remove_duplicates[1])):
			if i==1:
				data2 = np.vstack([data2, data[remove_duplicates[1][i]]]).transpose()
			else:
				data2 = np.vstack([data2.transpose(), data[remove_duplicates[1][i]]]).transpose()
		data2 = np.core.records.fromarrays(list(data2.transpose()))
		data2, indices = np.unique(data2, return_index=True)
		indices = np.sort(indices)
		data = data[indices]

	return data
