# -*- coding: iso_8859_1 -*-

from aspylib import astro, conf
import numpy as np

def coalign_coadd_and_save(Imagelist, mult, coadded_Image, Config, detection_area=10, nb_stars=12, excl=[], incl=[], text=False, method='mean', Headers={}):

	#--- loading images ---
	image0 = astro.get_imagedata([Imagelist[0]])
	print 'shifts of image 1 wrt image 1 (vert, hor):', 0., 0.	

	images_aligned = np.zeros([len(Imagelist)]+list(np.shape(image0)))
	images_aligned[0,:,:] = image0

	stars0 = astro.find_stars_accurate(image0, nb_stars, detection_area, excluded=excl, included=incl)
	for i in range(1,len(Imagelist)):
		
		#--- loading images ---
		imagei = astro.get_imagedata([Imagelist[i]])	

		#calculate shifts
		starsi = astro.find_stars_accurate(imagei, nb_stars, detection_area, excluded=excl, included=incl)
		dx,dy = astro.find_shift_stars_matching(stars0, starsi, text_display=text)
		print 'shifts of image', str(i+1),' wrt image 1 (vert, hor):', dx, dy

		#shift image
		images_aligned[i,:,:] = astro.translate_Bspline(imagei, (-dx,-dy), 3)

	if method == 'mean':
		image_final = mult*astro.add_mean(images_aligned)
	elif method == 'median':
		image_final = mult*astro.add_median(images_aligned)
	elif method == 'sigma clipping':
		image_final = mult*astro.add_sigma_clipping(images_aligned,3.,1)

	#save image
	if astro.get_headervalues([Imagelist[0]], 'EXPOSURE')[0]!='':
		Exp = np.sum(np.array(astro.get_headervalues(Imagelist, 'EXPOSURE')))
	else:
		Exp = np.sum(np.array(astro.get_headervalues(Imagelist, 'EXPTIME')))
	Dateobs = sorted(astro.get_headervalues(Imagelist, 'DATE-OBS'))[0]
	Dateend = sorted(astro.get_headervalues(Imagelist, 'DATE-END'))[-1]
	headerdict = astro.create_headers(Config, [Imagelist[0]])[0]
	try:
		del headerdict['EXPTIME']
	except KeyError:
		pass
	headerdict.update(Headers)
	headerdict.update({'EXPOSURE':[Exp,'Exposure (s)'], 'DATE-OBS':[Dateobs,''],'DATE-END':[Dateend,'']})
	astro.save_imagelist(image_final, [headerdict], [coadded_Image])
	print
	print coadded_Image + ' saved'
		
	return image_final

def coalign2_coadd_and_save(Imagelist, mult, coadded_Image, Config, method='mean', Headers={}):

	#--- loading images ---
	image0 = astro.get_imagedata([Imagelist[0]])
	print
	print 'shifts of image 1 wrt image 1 (vert, hor):', 0., 0.	

	images_aligned = np.zeros([len(Imagelist)]+list(np.shape(image0)))
	images_aligned[0,:,:] = image0

	for i in range(1,len(Imagelist)):
		
		#--- loading images ---
		imagei = astro.get_imagedata([Imagelist[i]])	

		#calculate shifts
		dx, dy = astro.find_shift_crosscorrelation(image0, imagei)
		print 'shifts of image', str(i+1),' wrt image 1 (vert, hor):', dx, dy

		#shift image
		images_aligned[i,:,:] = astro.translate_Bspline(imagei, (-dx,-dy), 3)

	if method == 'mean':
		image_final = mult*astro.add_mean(images_aligned)
	elif method == 'median':
		image_final = mult*astro.add_median(images_aligned)
	elif method == 'sigma clipping':
		image_final = mult*astro.add_sigma_clipping(images_aligned,3.,1)

	#save image
	if astro.get_headervalues([Imagelist[0]], 'EXPOSURE')[0]!='':
		Exp = np.sum(np.array(astro.get_headervalues(Imagelist, 'EXPOSURE')))
	else:
		Exp = np.sum(np.array(astro.get_headervalues(Imagelist, 'EXPTIME')))
	Dateobs = sorted(astro.get_headervalues(Imagelist, 'DATE-OBS'))[0]
	Dateend = sorted(astro.get_headervalues(Imagelist, 'DATE-END'))[-1]
	headerdict = astro.create_headers(Config, [Imagelist[0]])[0]
	try:
		del headerdict['EXPTIME']
	except KeyError:
		pass
	headerdict.update(Headers)
	headerdict.update({'EXPOSURE':[Exp,'Exposure (s)'], 'DATE-OBS':[Dateobs,''],'DATE-END':[Dateend,'']})
	astro.save_imagelist(image_final, [headerdict], [coadded_Image])
	print
	print coadded_Image + ' saved'
		
	return image_final
