# -*- coding: iso_8859_1 -*-

import os, datetime
import numpy as np
from aspylib import astro

def write_auto_magnitudes(Imagelist, xy, sizes, ref_stars, dateobs, mag, stats, quality_flag, which_disk, outputname, astrometry = []):
	
	if os.path.exists(outputname):
		os.remove(outputname)
	f = open(outputname, 'w')

	#--- nb of stars to be written to file ---
	ind = list(np.where(np.array(quality_flag)>=0)[0])
	
	#--- file header ---
	f.write('#--- MAIN INFORMATION ------------------\n')
	f.write('date = '+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
	f.write(str(len(Imagelist))+' images processed\n')
	f.write(str(len(quality_flag))+' objects detected\n')
	f.write(str(len(ind))+' objects successfully measured\n')
	f.write('first image = '+(Imagelist[0]).encode('iso_8859_1')+'\n')
	f.write('datation at start of exposure\n')
	h1 = astro.get_headervalues([Imagelist[0]],'EXPOSURE')[0]
	h2 = astro.get_headervalues([Imagelist[0]],'EXPTIME')[0]
	if h1!='':
		f.write('exposure time (sec):\t'+str(int(np.round(h1)))+'\n')
	elif h2!='':
		f.write('exposure time (sec):\t'+str(int(np.round(h2)))+'\n')
	f.write('photometry with apertures:\n')
	for i in range(len(sizes)):	
		f.write('disk_'+str(i)+'\t'+str(sizes[i][0])+' - '+str(sizes[i][1])+' - '+str(sizes[i][2])+'\n')
	f.write('\n')

	#--- reference stars ---
	#print ref_stars
	f.write('#--- REFERENCE STARS --\n')
	for i in range(len(sizes)):	
		f.write('disk_'+str(i)+':\n')
		f.write('nb of ref stars:' + str(len(ref_stars[i,:])) + '\n')
		f.write('ref stars:')
		for ref in np.sort(ref_stars[i,:]):
			f.write('\t' + str(ref+1))
		f.write('\n')
	f.write('\n')

	#--- stars X,Y coordinates ---
	f.write('#--- OBJECTS COORDINATES ON FIRST FRAME --\n')
	f.write('Object nb')
	for j in range(len(ind)):
		f.write('\t'+str(j+1))
	f.write('\n')
	f.write('X position (vertical)')
	for j in ind:
		f.write('\t'+"%.4f"%xy[0][j][0])
	f.write('\n')
	f.write('Y position (horizontal)')
	for j in ind:
		f.write('\t'+"%.4f"%xy[0][j][1])
	f.write('\n\n')

	#--- astrometry ---
	positions = astrometry[0]
	f.write('#--- ASTROMETRY --\n')
	f.write('alpha')
	for j in ind:
		f.write('\t'+astro.decdegrees_to_hours(positions[j][0]))
	f.write('\n')
	f.write('delta')
	for j in ind:
		f.write('\t'+astro.decdegrees_to_degrees(positions[j][1]))
	f.write('\n\n')

	#--- catalog positions ---
	usnob1 = astrometry[1]
	if len(usnob1)>0:
		f.write('#--- MATCHING WITH USNO-B1 --\n')
		f.write('distance (arcsec)')
		for j in ind:
			if len(usnob1[j])>2:
				alpha1 = positions[j][0]*np.pi/180.
				alpha2 = usnob1[j][1]*np.pi/180.
				delta1 = positions[j][1]*np.pi/180.
				delta2 = usnob1[j][2]*np.pi/180.
				dist = np.arccos(np.sin(delta1)*np.sin(delta2)+np.cos(delta1)*np.cos(delta2)*np.cos(alpha1-alpha2))*(180./np.pi)*3600.
				f.write('\t'+"%.2f"%dist)
			else:
				f.write('\t-')
		f.write('\n')
		f.write('name')
		for j in ind:
			if len(usnob1[j])>2:
				f.write('\t'+usnob1[j][0])
			else:
				f.write('\t-')
		f.write('\n')
		f.write('alpha')
		for j in ind:
			if len(usnob1[j])>2:
				f.write('\t'+astro.decdegrees_to_hours(usnob1[j][1]))
			else:
				f.write('\t-')
		f.write('\n')
		f.write('delta')
		for j in ind:
			if len(usnob1[j])>2:
				f.write('\t'+astro.decdegrees_to_degrees(usnob1[j][2]))
			else:
				f.write('\t-')
		f.write('\n')
		f.write('B1mag')
		for j in ind:
			if len(usnob1[j])>2:
				f.write('\t'+"%.2f"%usnob1[j][3])
			else:
				f.write('\t-')
		f.write('\n')
		f.write('B2mag')
		for j in ind:
			if len(usnob1[j])>2:
				f.write('\t'+"%.2f"%usnob1[j][4])
			else:
				f.write('\t-')
		f.write('\n')
		f.write('R1mag')
		for j in ind:
			if len(usnob1[j])>2:
				f.write('\t'+"%.2f"%usnob1[j][5])
			else:
				f.write('\t-')
		f.write('\n')
		f.write('R2mag')
		for j in ind:
			if len(usnob1[j])>2:
				f.write('\t'+"%.2f"%usnob1[j][6])
			else:
				f.write('\t-')
		f.write('\n\n')

	#--- statistics about photometry ---
	signal, noise_global, noise_local = stats

	f.write('#--- STATISTICS ------------------------\n')
	f.write('mag signal')
	for j in ind:
		f.write('\t'+"%.6f"%signal[j])
	f.write('\n')
	f.write('global noise')
	for j in ind:
		f.write('\t'+"%.6f"%noise_global[j])
	f.write('\n')
	f.write('local noise')
	for j in ind:
		f.write('\t'+"%.6f"%noise_local[j])
	f.write('\n')
	f.write('quality flag')
	for j in ind:
		f.write('\t'+str(quality_flag[j]))
	f.write('\n')
	f.write('disk used for photometry')
	for j in ind:
		f.write('\t'+str(which_disk[j]))
	f.write('\n\n')
	
	#--- measured magnitudes ---
	f.write('#--- JULIAN DAY + MEASURED MAGNITUDES ---------------\n')
	data = np.ndarray((len(dateobs),len(ind)+1))
	data[:,0] = np.array(dateobs)
	data[:,1:] = np.ma.filled(mag[:,ind],fill_value=np.nan)
	np.savetxt(f,data,delimiter='\t',fmt='%.6f')

	f.close()
	#-------------------------------------